import tensorflow as tf
import numpy as np
import json

from tensorflow.python.tools import inspect_checkpoint as chkp
from tensorflow.python import pywrap_tensorflow

def get_weights(trainable_ops):
    o = {}
    for op in trainable_ops:
        if('kernel' in op.name):
            name = op.name.split("/")[0]
            o[name] = op
    return o

def get_activations(vis_ops):
    o = {}
    for op in vis_ops:
        if ('Relu' in op.name or 'maxpool' in op.name or 'out' in op.name ):
            name = op.name.split("/")[0]
            o[name] = op
    return o

def get_vis2_layers(vis_ops, weight_ops, weight_values):
    o = []
    input_shape = vis_ops[0].shape[1:3]
    o.append(('input', input_shape.as_list(), np.zeros(input_shape).tolist()))

    vis_op_names = set()

    for op in vis_ops:
        name = op.name.split("/")[0]
        if(name in vis_op_names):
            continue
        vis_op_names.add(name)
        if('maxpool' in op.name):
            o.append((name, op.shape[1:3].as_list(), np.zeros(op.shape[1:3].as_list()).tolist()))
        elif(name in weight_ops.keys()):
            shape = weight_ops[name].shape.as_list()
            weights = weight_values[name]
            o.append((name, shape, weights.tolist()))

    return o

b_test = np.load('dataset/batch_test.npy')

x_test = b_test[:, 1:]
y_test = b_test[:, 0]
x_test = np.reshape(x_test, [-1, 32, 32, 1])

dataset_test = tf.data.Dataset.from_tensor_slices((x_test, y_test)).repeat().shuffle(buffer_size=500).batch(1)

for i in range(10):
    with tf.Session() as sess:
        new_saver = tf.train.import_meta_graph('ckpt/model.ckpt-{}.meta'.format(i))
        new_saver.restore(sess, 'ckpt/model.ckpt-{}'.format(i))

        print(sess.run(tf.report_uninitialized_variables()))

        dataset_init_op = sess.graph.get_operation_by_name('dataset_init')



        logits = model.inference(features)

        vis_ops = tf.get_collection('VisOps')
        train_ops = tf.get_collection(tf.GraphKeys.TRAINABLE_VARIABLES)
        weight_ops = get_weights(train_ops)
        weight_values = sess.run(weight_ops)
        activation_values = sess.run(get_activations(vis_ops))

        json_base = get_vis2_layers(vis_ops, weight_ops, weight_values)

        with open("epoch{}.json".format(i), "w") as outfile:
            try:
                json.dump(json_base, outfile)
            except Exception as e:
                print(e.message)